import { HoliEvent } from '../domain/HoliEvent.ts'
import { Logs } from './dependencies/Logs.ts'
import { Activity } from '../adapters/jasd/types/appapi.dto.types.ts'
import { FetchesJasdActivity } from './dependencies/FetchesJasdActivity.ts'

export interface QueryEventInput {
  id: string
}

export interface QueryEventOutput {
  data: HoliEvent
}

const APP_FILES_BASE_URL = 'https://gemeinschaftswerk-nachhaltigkeit.de/app/api/v1/files'

export class QueryEvent {
  constructor(
    private readonly jasdApi: FetchesJasdActivity,
    private readonly logger: Logs,
  ) {}

  async execute(input: QueryEventInput): Promise<QueryEventOutput> {
    this.logger.debug(`Executing QueryEvent use case with input: ${JSON.stringify(input)}`)
    const startTime = Date.now()

    try {
      const jasdActivity = await this.jasdApi.fetchActivity(
        input.id,
      )

      const holiEvent = this.toDomainEvent(jasdActivity)
      return { data: holiEvent }
    } catch (error) {
      this.logger.error('Error executing QueryEvent use case', error)
      throw error
    } finally {
      const duration = Date.now() - startTime
      this.logger.info(`QueryEvent use case executed in ${duration}ms`)
    }
  }

  //todo montagsgregor: move to gateway as its duplicated
  private toDomainEvent(activity: Activity): HoliEvent {
    return {
      id: activity.id.toString(),
      title: activity.name,
      organisationName: activity.organisation.name,
      startDate: activity.period.start,
      endDate: activity.period.end,
      location: {
        city: activity.location.address.city,
        street: activity.location.address.street,
        streetNo: activity.location.address.streetNo,
        zipCode: activity.location.address.zipCode,
      },
      isRemote: activity.location.online,
      imageUrl: `${APP_FILES_BASE_URL}/${activity.image}`,
      eventDetails: {
        description: activity.description,
        externalLink: activity.location.url,
        meetingLink: activity.registerUrl ?? undefined,
      },
    }
  }
}
