import { assertEquals } from '@std/assert'
import { describe, it } from '@std/testing/bdd'
import { returnsNext, stub } from '@std/testing/mock'

import { Address, Location } from '../adapters/jasd/types/appapi.dto.types.ts'
import { LocalTimedActivity, OnlineTimedActivity } from '../adapters/jasd/tests/fixtures.ts'
import { JasdGateway } from '../adapters/jasd/JasdGateway.ts'
import { QueryNearbyEvents } from './QueryNearbyEvents.ts'

describe('QueryNearbyEvents', () => {
  const geoApiAMock = {
    resolvePlace() {
      return Promise.resolve({ name: 'Village', city: 'Municipality', state: 'State' })
    },
  }

  describe('when requesting 5 events', () => {
    it('it puts a events in the users city first in the response', async () => {
      const activitiesForCityResponse = {
        events: [
          {
            name: 'a',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: aLocation({ address: anAddress({ city: 'Municipality' }) }),
          },
          {
            name: 'b',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: aLocation({ address: anAddress({ city: 'Municipality' }) }),
          },
        ],
        totalCount: 2,
      }

      const activitiesForStateResponse = {
        events: [
          {
            name: 'c',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: aLocation({ address: anAddress({ state: 'State' }) }),
          },
        ],
        totalCount: 1,
      }
      const activitiesUnfilteredResponse = {
        events: [
          {
            name: 'd',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: LocalTimedActivity,
            location: LocalTimedActivity.location,
          },
          {
            name: 'e',
            description: 'description',
            resultType: 'ACTIVITY',
            activity: OnlineTimedActivity,
            location: OnlineTimedActivity.location,
          },
        ],
        totalCount: 2,
      }

      const fetchActivitiesSpy = stub(
        {} as JasdGateway,
        'fetchActivities',
        returnsNext([
          Promise.resolve(activitiesForCityResponse),
          Promise.resolve(activitiesForStateResponse),
          Promise.resolve(activitiesUnfilteredResponse),
        ]),
      )

      const sut = new QueryNearbyEvents(
        {
          fetchActivities: fetchActivitiesSpy,
        },
        geoApiAMock,
        console,
      )

      const result = await sut.execute({
        limit: 5,
        offset: 0,
        geolocationId: 'someGeolocationId',
      })

      assertEquals(result.data.length, 5)
      assertEquals(result.data[0].title, 'a')
      assertEquals(result.data[1].title, 'b')
      assertEquals(result.data[2].title, 'c')
      assertEquals(result.data[3].title, 'd')
      assertEquals(result.data[4].title, 'e')
    })
  })
})

// todo gregor: test remote events in the location-based set are ignored

const aLocation = (override?: Partial<Location>): Location => {
  const fallback: Location = {
    address: anAddress(),
    coordinate: {
      coordinates: [0, 0],
      type: 'Point',
    },
    online: false,
    privateLocation: false,
    url: '',
  }

  return { ...fallback, ...override }
}

const anAddress = (override?: Partial<Address>): Address => {
  const fallback = {
    city: '',
    country: null,
    name: '',
    state: null,
    street: '',
    streetNo: '',
    supplement: null,
    zipCode: '',
  }
  return { ...fallback, ...override }
}
